#include <math.h>

#include "fieldsat.h"

int press_low = 0, press_high = 0;

int main(int argc, char *argv[])
{
	struct fieldsat_sample sample;
	FILE *fp;
	long int n_samples, n_bytes;
	int i;
	char buf[500];
	unsigned long previous_timestamp = 0;
	double t, time, previous_time;

	fp = stream_open(argc, argv);

	time = 0.0;
	previous_time = 0.0;
	n_samples = 0;
	while (read_sample(fp, &sample)) {
		if (n_samples == 0) {
			t = 0;
		} else {
			t = (double) (sample.timestamp - previous_timestamp) / F_CPU;
		}
		n_samples++;
		if (t > 1.0)
			printf("Warning: t = %f at %02d:%02d:%02.3f %lu %lu.\n", t, (int) (time / 3600), (int) ((fmod(time, 3600)) / 60), fmod(time, 60), previous_timestamp, sample.timestamp);
		time += t;
		previous_timestamp = sample.timestamp;
		if (time - previous_time > 15 * 60) {
			printf("%f samples/second %lu\n", n_samples / (time - previous_time), ftell(fp));
			previous_time += 15 * 60;
			n_samples = 0;
		}
		if (sample.type == SAMPLE_TYPE_S2) {
			/*
			if (sample.s2.pressure != press_high || sample.s2.temp_int != press_low) {
				press_high = sample.s2.pressure;
				press_low = sample.s2.temp_int;
				printf("%d %d\n", press_low, press_high);
			}
			*/
			/*
			if (sample.s2.temp_int != 511) {
				printf("temp_int = %d at t = %f.\n", (int) sample.s2.temp_int, time);
			}
			*/
		}
	}

	n_bytes = ftell(fp);
	if (!feof(fp))
		n_bytes -= FIELDSAT_SAMPLE_SIZE;

	printf("%f sectors\n", n_bytes / 512.0);
	printf("%02d:%02d:%02.3f\n", (int) (time / 3600), (int) ((fmod(time, 3600)) / 60), fmod(time, 60));

	fread(buf, sizeof(buf), 1, fp);
	if (!feof(fp)) {
		for (i = 0; i < sizeof(buf); i++) {
			if (buf[i] != 0x00)
				printf("%d ", i);
		}
		putchar('\n');
	}

	return 0;
}
