#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include "fieldsat.h"

#define NUM_SAMPLES 3000

#define MIN(a, b) ((a) < (b) ? (a) : (b))
#define MAX(a, b) ((a) > (b) ? (a) : (b))
#define SQUARE(x) ((x) * (x))

volatile int terminate = 0;

void sigterm(int signum)
{
	terminate = 1;
}

#ifdef main
#undef main
#endif

int main(int argc, char *argv[])
{
	struct fieldsat_sample sample, accum, accum_squared;
	struct fieldsat_sample min, max;
	unsigned long n;
	FILE *fp;

	fp = stream_open(argc, argv);

	signal(SIGINT, sigterm);

	sample_init(&sample);
	sample_init(&accum);
	sample_init(&accum_squared);
	sample_init(&min);
	sample_init(&max);

	min.s1.acc_low[0] = 10000.0;
	min.s1.acc_low[1] = 10000.0;
	min.s1.acc_low[2] = 10000.0;
	min.s1.ang_vel[0] = 10000.0;
	min.s1.ang_vel[1] = 10000.0;
	min.s1.ang_vel[2] = 10000.0;
	min.s1.solar_luminosity = 10000.0;
	min.s1.solar_voltage = 10000.0;

	if (!synchronize(fp)) {
		fprintf(stderr, "Couldn't synchronize to stream.\n");
		return 1;
	}

	for (n = 0; n < NUM_SAMPLES && !terminate; n++) {
		if (!read_sample(fp, &sample))
			break;

		accum.s1.acc_low[0] += sample.s1.acc_low[0];
		accum.s1.acc_low[1] += sample.s1.acc_low[1];
		accum.s1.acc_low[2] += sample.s1.acc_low[2];
		accum.s1.ang_vel[0] += sample.s1.ang_vel[0];
		accum.s1.ang_vel[1] += sample.s1.ang_vel[1];
		accum.s1.ang_vel[2] += sample.s1.ang_vel[2];
		accum.s1.solar_luminosity += sample.s1.solar_luminosity;
		accum.s1.solar_voltage += sample.s1.solar_voltage;

		accum_squared.s1.acc_low[0] += SQUARE(sample.s1.acc_low[0]);
		accum_squared.s1.acc_low[1] += SQUARE(sample.s1.acc_low[1]);
		accum_squared.s1.acc_low[2] += SQUARE(sample.s1.acc_low[2]);
		accum_squared.s1.ang_vel[0] += SQUARE(sample.s1.ang_vel[0]);
		accum_squared.s1.ang_vel[1] += SQUARE(sample.s1.ang_vel[1]);
		accum_squared.s1.ang_vel[2] += SQUARE(sample.s1.ang_vel[2]);
		accum_squared.s1.solar_luminosity += SQUARE(sample.s1.solar_luminosity);
		accum_squared.s1.solar_voltage += SQUARE(sample.s1.solar_voltage);

		min.s1.acc_low[0] = MIN(min.s1.acc_low[0], sample.s1.acc_low[0]);
		min.s1.acc_low[1] = MIN(min.s1.acc_low[1], sample.s1.acc_low[1]);
		min.s1.acc_low[2] = MIN(min.s1.acc_low[2], sample.s1.acc_low[2]);
		min.s1.ang_vel[0] = MIN(min.s1.ang_vel[0], sample.s1.ang_vel[0]);
		min.s1.ang_vel[1] = MIN(min.s1.ang_vel[1], sample.s1.ang_vel[1]);
		min.s1.ang_vel[2] = MIN(min.s1.ang_vel[2], sample.s1.ang_vel[2]);
		min.s1.solar_luminosity = MIN(min.s1.solar_luminosity, sample.s1.solar_luminosity);
		min.s1.solar_voltage = MIN(min.s1.solar_voltage, sample.s1.solar_voltage);

		max.s1.acc_low[0] = MAX(max.s1.acc_low[0], sample.s1.acc_low[0]);
		max.s1.acc_low[1] = MAX(max.s1.acc_low[1], sample.s1.acc_low[1]);
		max.s1.acc_low[2] = MAX(max.s1.acc_low[2], sample.s1.acc_low[2]);
		max.s1.ang_vel[0] = MAX(max.s1.ang_vel[0], sample.s1.ang_vel[0]);
		max.s1.ang_vel[1] = MAX(max.s1.ang_vel[1], sample.s1.ang_vel[1]);
		max.s1.ang_vel[2] = MAX(max.s1.ang_vel[2], sample.s1.ang_vel[2]);
		max.s1.solar_luminosity = MAX(max.s1.solar_luminosity, sample.s1.solar_luminosity);
		max.s1.solar_voltage = MAX(max.s1.solar_voltage, sample.s1.solar_voltage);
	}

	accum.s1.acc_low[0] /= n;
	accum.s1.acc_low[1] /= n;
	accum.s1.acc_low[2] /= n;
	accum.s1.ang_vel[0] /= n;
	accum.s1.ang_vel[1] /= n;
	accum.s1.ang_vel[2] /= n;
	accum.s1.solar_luminosity /= n;
	accum.s1.solar_voltage /= n;

	accum_squared.s1.acc_low[0] /= n;
	accum_squared.s1.acc_low[1] /= n;
	accum_squared.s1.acc_low[2] /= n;
	accum_squared.s1.ang_vel[0] /= n;
	accum_squared.s1.ang_vel[1] /= n;
	accum_squared.s1.ang_vel[2] /= n;
	accum_squared.s1.solar_luminosity /= n;
	accum_squared.s1.solar_voltage /= n;

	/*
	*/
	printf("n: %lu\n", n);

	printf("Acc. low  X min: %03.1f  max: %03.1f  mean: %04f  variance: %04f\n", min.s1.acc_low[0], max.s1.acc_low[0], accum.s1.acc_low[0], accum_squared.s1.acc_low[0] - SQUARE(accum.s1.acc_low[0]));
	printf("Acc. low  Y min: %03.1f  max: %03.1f  mean: %04f  variance: %04f\n", min.s1.acc_low[1], max.s1.acc_low[1], accum.s1.acc_low[1], accum_squared.s1.acc_low[1] - SQUARE(accum.s1.acc_low[1]));
	printf("Acc. low  Z min: %03.1f  max: %03.1f  mean: %04f  variance: %04f\n", min.s1.acc_low[2], max.s1.acc_low[2], accum.s1.acc_low[2], accum_squared.s1.acc_low[2] - SQUARE(accum.s1.acc_low[2]));
	printf("Ang. vel. X min: %03.1f  max: %03.1f  mean: %04f  variance: %04f\n", min.s1.ang_vel[0], max.s1.ang_vel[0], accum.s1.ang_vel[0], accum_squared.s1.ang_vel[0] - SQUARE(accum.s1.ang_vel[0]));
	printf("Ang. vel. Y min: %03.1f  max: %03.1f  mean: %04f  variance: %04f\n", min.s1.ang_vel[1], max.s1.ang_vel[1], accum.s1.ang_vel[1], accum_squared.s1.ang_vel[1] - SQUARE(accum.s1.ang_vel[1]));
	printf("Ang. vel. Z min: %03.1f  max: %03.1f  mean: %04f  variance: %04f\n", min.s1.ang_vel[2], max.s1.ang_vel[2], accum.s1.ang_vel[2], accum_squared.s1.ang_vel[2] - SQUARE(accum.s1.ang_vel[2]));
	printf("Sol. lum. min: %03.1f  max: %03.1f  mean: %04f  variance: %04f\n", min.s1.solar_luminosity, max.s1.solar_luminosity, accum.s1.solar_luminosity, accum_squared.s1.solar_luminosity - SQUARE(accum.s1.solar_luminosity));
	printf("Sol. vol. min: %03.1f  max: %03.1f  mean: %04f  variance: %04f\n", min.s1.solar_voltage, max.s1.solar_voltage, accum.s1.solar_voltage, accum_squared.s1.solar_voltage - SQUARE(accum.s1.solar_voltage));

	printf("\n");

	printf("const double ACC_LOW_ZERO[3] = { %.8f, %.8f, %.8f };\n", accum.s1.acc_low[0], accum.s1.acc_low[1], accum.s1.acc_low[2]);
	printf("const double ACC_LOW_VARIANCE[3] = { %.8f, %.8f, %.8f };\n", accum_squared.s1.acc_low[0] - SQUARE(accum.s1.acc_low[0]), accum_squared.s1.acc_low[1] - SQUARE(accum.s1.acc_low[1]), accum_squared.s1.acc_low[2] - SQUARE(accum.s1.acc_low[2]));
	printf("const double ANG_VEL_ZERO[3] = { %.8f, %.8f, %.8f };\n", accum.s1.ang_vel[0], accum.s1.ang_vel[1], accum.s1.ang_vel[2]);
	printf("const double ANG_VEL_VARIANCE[3] = { %.8f, %.8f, %.8f };\n", accum_squared.s1.ang_vel[0] - SQUARE(accum.s1.ang_vel[0]), accum_squared.s1.ang_vel[1] - SQUARE(accum.s1.ang_vel[1]), accum_squared.s1.ang_vel[2] - SQUARE(accum.s1.ang_vel[2]));

	return 0;
}
